@@ -207,7 +207,7 @@ package Sidef::Types::Array::Array {
             $addr{$refaddr} = bless \@array;
 
             foreach my $item (@$obj) {
-                if (ref($item) eq __PACKAGE__) {
+                if (ref($item) eq __PACKAGE__ or UNIVERSAL::isa($item, __PACKAGE__)) {
                     CORE::push(@array, __SUB__->($item));
                 }
                 else {
@@ -246,7 +246,7 @@ package Sidef::Types::Array::Array {
             $addr{$refaddr} = bless \@array;
 
             foreach my $item (@$obj) {
-                if (ref($item) eq __PACKAGE__) {
+                if (ref($item) eq __PACKAGE__ or UNIVERSAL::isa($item, __PACKAGE__)) {
                     CORE::push(@array, __SUB__->($item));
                 }
                 else {
@@ -290,7 +290,7 @@ package Sidef::Types::Array::Array {
             $addr{$refaddr2} = $addr{$refaddr1} = bless \@array;
 
             for my $i (0 .. $#{$obj1}) {
-                if (ref($obj1->[$i]) eq __PACKAGE__) {
+                if (ref($obj1->[$i]) eq __PACKAGE__ or UNIVERSAL::isa($obj1->[$i], __PACKAGE__)) {
                     CORE::push(@array, __SUB__->($obj1->[$i], $obj2->[$i]));
                 }
                 else {
@@ -310,6 +310,47 @@ package Sidef::Types::Array::Array {
 
     *wise_op = \&wise_operator;
 
+    sub combine {
+        my ($self, $block) = @_;
+
+        my %addr;    # support for cyclic references
+
+        sub {
+            my (@arrays) = @_;
+
+            my @array;
+            my $blessed_array = bless \@array;
+
+            # Check any references already computed
+            foreach my $obj (@arrays) {
+                my $refaddr = Scalar::Util::refaddr($obj);
+                exists($addr{$refaddr}) && return $addr{$refaddr};
+            }
+
+            # Store the references of each object
+            foreach my $obj (@arrays) {
+                my $refaddr = Scalar::Util::refaddr($obj);
+                $addr{$refaddr} = $blessed_array;
+            }
+
+            @arrays || return $blessed_array;
+
+            my $first = $arrays[0];
+
+            foreach my $i (0 .. $#{$first}) {
+                if (ref($first->[$i]) eq __PACKAGE__ or UNIVERSAL::isa($first->[$i], __PACKAGE__)) {
+                    CORE::push(@array, __SUB__->(map { $_->[$i] } @arrays));
+                }
+                else {
+                    CORE::push(@array, $block->run(map { $_->[$i] } @arrays));
+                }
+            }
+
+            $blessed_array;
+          }
+          ->(@$self);
+    }
+
     sub mul {
         my ($self, $num) = @_;
         bless [(@$self) x CORE::int($num)];
@@ -432,13 +473,6 @@ package Sidef::Types::Array::Array {
         bless \@new;
     }
 
-    sub is_empty {
-        my ($self) = @_;
-        ($#$self < 0)
-          ? (Sidef::Types::Bool::Bool::TRUE)
-          : (Sidef::Types::Bool::Bool::FALSE);
-    }
-
     sub sub {
         my ($self, $array) = @_;
 
@@ -515,7 +549,7 @@ package Sidef::Types::Array::Array {
     sub concat {
         my ($self, $arg) = @_;
 
-        UNIVERSAL::isa($arg, 'ARRAY')
+        ref($self) eq ref($arg)
           ? bless([@$self, @$arg])
           : bless([@$self, $arg]);
     }
@@ -605,29 +639,36 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Number::Number->new($jaro + $prefix * 0.1 * (1 - $jaro));
     }
 
-    sub count {
-        my ($self, $obj) = @_;
+    sub count_by {
+        my ($self, $block) = @_;
 
         my $counter = 0;
-        if (ref($obj) eq 'Sidef::Types::Block::Block') {
-
-            foreach my $item (@$self) {
-                if ($obj->run($item)) {
-                    ++$counter;
-                }
-            }
-
-            return Sidef::Types::Number::Number->_set_uint($counter);
-        }
 
         foreach my $item (@$self) {
-            $item eq $obj and $counter++;
+            if ($block->run($item)) {
+                ++$counter;
+            }
         }
 
         Sidef::Types::Number::Number->_set_uint($counter);
     }
 
-    *count_by = \&count;
+    sub count {
+        my ($self, $obj) = @_;
+
+        if (ref($obj) eq 'Sidef::Types::Block::Block') {
+            goto &count_by;
+        }
+
+        my $counter = 0;
+        foreach my $item (@$self) {
+            if ($item eq $obj) {
+                ++$counter;
+            }
+        }
+
+        Sidef::Types::Number::Number->_set_uint($counter);
+    }
 
     sub cmp {
         my ($self, $array) = @_;
@@ -895,6 +936,48 @@ package Sidef::Types::Array::Array {
         Sidef::Math::Math->prod(@$self);
     }
 
+    sub gcd_by {
+        my ($self, $block) = @_;
+
+        my @list;
+        foreach my $n (@$self) {
+            push @list, $block->run($n);
+        }
+
+        Sidef::Math::Math->gcd(@list);
+    }
+
+    sub gcd {
+        my ($self, $block) = @_;
+
+        if (defined($block)) {
+            goto &gcd_by;
+        }
+
+        Sidef::Math::Math->gcd(@$self);
+    }
+
+    sub lcm_by {
+        my ($self, $block) = @_;
+
+        my @list;
+        foreach my $n (@$self) {
+            push @list, $block->run($n);
+        }
+
+        Sidef::Math::Math->lcm(@list);
+    }
+
+    sub lcm {
+        my ($self, $block) = @_;
+
+        if (defined($block)) {
+            goto &lcm_by;
+        }
+
+        Sidef::Math::Math->lcm(@$self);
+    }
+
     sub _min_max_by {
         my ($self, $block, $order) = @_;
 
@@ -1232,7 +1315,7 @@ package Sidef::Types::Array::Array {
         foreach my $item (@copy) {
             my $res = $block->run($item);
 
-            if (ref($res) eq __PACKAGE__) {
+            if (ref($res) eq __PACKAGE__ or UNIVERSAL::isa($res, __PACKAGE__)) {
                 CORE::push(@copy, @$res);
             }
             else {
@@ -1257,7 +1340,7 @@ package Sidef::Types::Array::Array {
         foreach my $item (@copy) {
             my $res = $block->run($item);
 
-            if (ref($res) eq __PACKAGE__) {
+            if (ref($res) eq __PACKAGE__ or UNIVERSAL::isa($res, __PACKAGE__)) {
                 CORE::push(@copy, @$res);
             }
         }
@@ -2414,6 +2497,19 @@ package Sidef::Types::Array::Array {
         bless [@$self[0 .. $amount]];
     }
 
+    sub is_empty {
+        my ($self) = @_;
+        ($#$self < 0)
+          ? (Sidef::Types::Bool::Bool::TRUE)
+          : (Sidef::Types::Bool::Bool::FALSE);
+    }
+
+    sub clear {
+        my ($self) = @_;
+        @$self = ();
+        $self;
+    }
+
     sub sort {
         my ($self, $block) = @_;
 
@@ -2599,224 +2695,37 @@ package Sidef::Types::Array::Array {
 
     sub det_bareiss {
         my ($self) = @_;
-
-        my @m = map { [@$_] } @$self;
-
-        my $neg   = 0;
-        my $pivot = Sidef::Types::Number::Number::ONE;
-        my $end   = $#m;
-
-        foreach my $k (0 .. $end) {
-            my @r = ($k + 1 .. $end);
-
-            my $prev_pivot = $pivot;
-            $pivot = $m[$k][$k] // return Sidef::Types::Number::Number::ONE;
-
-            if ($pivot eq Sidef::Types::Number::Number::ZERO) {
-                my $i = List::Util::first(sub { $m[$_][$k] }, @r) // return Sidef::Types::Number::Number::ZERO;
-                @m[$i, $k] = @m[$k, $i];
-                $pivot = $m[$k][$k];
-                $neg ^= 1;
-            }
-
-            foreach my $i (@r) {
-                foreach my $j (@r) {
-                    $m[$i][$j] = $m[$i][$j]->mul($pivot);
-                    $m[$i][$j] = $m[$i][$j]->sub($m[$i][$k]->mul($m[$k][$j]));
-                    $m[$i][$j] = $m[$i][$j]->div($prev_pivot);
-                }
-            }
-        }
-
-        $neg ? $pivot->neg : $pivot;
+        $self->to_matrix->det_bareiss;
     }
 
     # Reduced row echelon form
     sub rref {
         my ($self) = @_;
-
-        my @m = map { [@$_] } @$self;
-
-        @m || return Sidef::Types::Array::Array->new();
-
-        my ($j, $rows, $cols) = (0, scalar(@m), scalar(@{$m[0]}));
-
-      OUTER: foreach my $r (0 .. $rows - 1) {
-
-            $j < $cols or last;
-
-            my $i = $r;
-
-            while ($m[$i][$j]->is_zero) {
-                ++$i == $rows or next;
-                $i = $r;
-                ++$j == $cols and last OUTER;
-            }
-
-            @m[$i, $r] = @m[$r, $i];
-
-            my $t = $m[$r][$j];
-            foreach my $k (0 .. $cols - 1) {
-                $m[$r][$k] = $m[$r][$k]->div($t);
-            }
-
-            foreach my $i (0 .. $rows - 1) {
-
-                $i == $r and next;
-
-                my $t = $m[$i][$j];
-                foreach my $k (0 .. $cols - 1) {
-                    $m[$i][$k] = $m[$i][$k]->sub($t->mul($m[$r][$k]));
-                }
-            }
-
-            ++$j;
-        }
-
-        bless $_ for @m;
-        bless \@m;
+        $self->to_matrix->rref;
     }
 
     *reduced_row_echelon_form = \&rref;
 
     sub gauss_jordan_invert {
         my ($self) = @_;
-
-        my $n = $#$self;
-
-#<<<
-        my @I = map {
-            my $i = $_;
-            [map {
-                $i == $_
-                    ? Sidef::Types::Number::Number::ONE
-                    : Sidef::Types::Number::Number::ZERO
-            } 0 .. $n]
-        } 0 .. $n;
-#>>>
-
-        my @A = map { [@{$self->[$_]}, @{$I[$_]}] } 0 .. $n;
-
-        my $r = rref(\@A);
-        @A = map { bless [@{$_}[$n + 1 .. $#$_]] } @$r;
-        bless \@A;
+        $self->to_matrix->gauss_jordan_invert;
     }
 
     sub gauss_jordan_solve {
         my ($self, $vector) = @_;
-
-        my @A = map { [@{$self->[$_]}, $vector->[$_]] } 0 .. $#$vector;
-
-        my $r = rref(\@A);
-        bless [map { $_->[-1] } @$r];
-    }
-
-    # Code translated from Wikipedia (+ minor tweaks):
-    #   https://en.wikipedia.org/wiki/LU_decomposition#C_code_examples
-
-    sub _LUP_decompose {
-        my ($self) = @_;
-
-        my @A = map { [@$_] } @$self;
-        my $N = $#A;
-        my @P = (0 .. $N + 1);
-
-        foreach my $i (0 .. $N) {
-
-            my $maxA = Sidef::Types::Number::Number::ZERO;
-            my $imax = $i;
-
-            foreach my $k ($i .. $N) {
-                my $absA = ($A[$k][$i] // return ($N, \@A, \@P))->abs;
-
-                if ($absA->gt($maxA)) {
-                    $maxA = $absA;
-                    $imax = $k;
-                }
-            }
-
-            if ($imax != $i) {
-
-                @P[$i, $imax] = @P[$imax, $i];
-                @A[$i, $imax] = @A[$imax, $i];
-
-                ++$P[$N + 1];
-            }
-
-            foreach my $j ($i + 1 .. $N) {
-
-                if ($A[$i][$i]->is_zero) {
-                    return ($N, \@A, \@P);
-                }
-
-                $A[$j][$i] = $A[$j][$i]->div($A[$i][$i]);
-
-                foreach my $k ($i + 1 .. $N) {
-                    $A[$j][$k] = $A[$j][$k]->sub($A[$j][$i]->mul($A[$i][$k]));
-                }
-            }
-        }
-
-        return ($N, \@A, \@P);
+        $self->to_matrix->gauss_jordan_solve($vector);
     }
 
     sub matrix_solve {
         my ($self, $vector) = @_;
-
-        my ($N, $A, $P) = $self->_LUP_decompose;
-
-        my @x = map { $vector->[$P->[$_]] } 0 .. $N;
-
-        foreach my $i (1 .. $N) {
-            foreach my $k (0 .. $i - 1) {
-                $x[$i] = $x[$i]->sub($A->[$i][$k]->mul($x[$k]));
-            }
-        }
-
-        for (my $i = $N ; $i >= 0 ; --$i) {
-            foreach my $k ($i + 1 .. $N) {
-                $x[$i] = $x[$i]->sub($A->[$i][$k]->mul($x[$k]));
-            }
-            $x[$i] = $x[$i]->div($A->[$i][$i]);
-        }
-
-        bless \@x;
+        $self->to_matrix->solve($vector);
     }
 
     *msolve = \&matrix_solve;
 
     sub invert {
         my ($self) = @_;
-
-        my ($N, $A, $P) = $self->_LUP_decompose;
-
-        my @I;
-
-        foreach my $j (0 .. $N) {
-            foreach my $i (0 .. $N) {
-
-                $I[$i][$j] = (
-                              ($P->[$i] == $j)
-                              ? Sidef::Types::Number::Number::ONE
-                              : Sidef::Types::Number::Number::ZERO
-                             );
-
-                foreach my $k (0 .. $i - 1) {
-                    $I[$i][$j] = $I[$i][$j]->sub($A->[$i][$k]->mul($I[$k][$j]));
-                }
-            }
-
-            for (my $i = $N ; $i >= 0 ; --$i) {
-                foreach my $k ($i + 1 .. $N) {
-                    $I[$i][$j] = $I[$i][$j]->sub($A->[$i][$k]->mul($I[$k][$j]));
-                }
-
-                $I[$i][$j] = $I[$i][$j]->div($A->[$i][$i] // return bless [bless []]);
-            }
-        }
-
-        bless $_ for @I;
-        bless \@I;
+        $self->to_matrix->invert;
     }
 
     *inv     = \&invert;
@@ -2824,58 +2733,39 @@ package Sidef::Types::Array::Array {
 
     sub determinant {
         my ($self) = @_;
-
-        my ($N, $A, $P) = $self->_LUP_decompose;
-
-        my $det = $A->[0][0] // return Sidef::Types::Number::Number::ONE;
-
-        foreach my $i (1 .. $N) {
-            $det = $det->mul($A->[$i][$i]);
-        }
-
-        if (($P->[$N + 1] - $N) % 2 == 0) {
-            $det = $det->neg;
-        }
-
-        return $det;
+        $self->to_matrix->det;
     }
 
     *det = \&determinant;
 
+    sub matrix_add {
+        my ($m1, $m2) = @_;
+        $m1->wise_operator('+', $m2);
+    }
+
+    *madd = \&matrix_add;
+
+    sub matrix_sub {
+        my ($m1, $m2) = @_;
+        $m1->wise_operator('-', $m2);
+    }
+
+    *msub = \&matrix_sub;
+
     sub matrix_mul {
         my ($m1, $m2) = @_;
-
-        my @a = map { [@$_] } @$m1;
-        my @b = map { [@$_] } @$m2;
-
-        my @c;
-
-        my $a_rows = $#a;
-        my $b_rows = $#b;
-        my $b_cols = $#{$b[0]};
-
-        foreach my $i (0 .. $a_rows) {
-            foreach my $j (0 .. $b_cols) {
-                foreach my $k (0 .. $b_rows) {
-
-                    my $t = $a[$i][$k]->mul($b[$k][$j]);
-
-                    if (!defined($c[$i][$j])) {
-                        $c[$i][$j] = $t;
-                    }
-                    else {
-                        $c[$i][$j] = $c[$i][$j]->add($t);
-                    }
-                }
-            }
-        }
-
-        bless $_ for @c;
-        bless \@c;
+        $m1->to_matrix->mul($m2);
     }
 
     *mmul = \&matrix_mul;
 
+    sub matrix_div {
+        my ($m1, $m2) = @_;
+        $m1->matrix_mul($m2->to_matrix->inv);
+    }
+
+    *mdiv = \&matrix_div;
+
     sub scalar_add {
         my ($self, $scalar) = @_;
         $self->scalar_operator('+', $scalar);
@@ -2904,53 +2794,9 @@ package Sidef::Types::Array::Array {
 
     *sdiv = \&scalar_div;
 
-    sub matrix_add {
-        my ($m1, $m2) = @_;
-        $m1->wise_operator('+', $m2);
-    }
-
-    *madd = \&matrix_add;
-
-    sub matrix_sub {
-        my ($m1, $m2) = @_;
-        $m1->wise_operator('-', $m2);
-    }
-
-    *msub = \&matrix_sub;
-
     sub matrix_pow {
         my ($A, $pow) = @_;
-
-        $pow = CORE::int($pow);
-
-        my $neg = 0;
-
-        if ($pow < 0) {
-            $neg = 1;
-            $pow = -$pow;
-        }
-
-#<<<
-        my $n = $#$A;
-        my $B = bless [map {
-            my $i = $_;
-            bless [map {
-                $i == $_
-                    ? Sidef::Types::Number::Number::ONE
-                    : Sidef::Types::Number::Number::ZERO
-            } 0 .. $n]
-        } 0 .. $n];
-#>>>
-
-        return $B if ($pow == 0);
-
-        while (1) {
-            $B = $B->mmul($A) if ($pow & 1);
-            $pow >>= 1 or last;
-            $A = $A->mmul($A);
-        }
-
-        $neg ? $B->inv : $B;
+        $A->to_matrix->pow($pow);
     }
 
     *mpow = \&matrix_pow;
@@ -3102,25 +2948,6 @@ package Sidef::Types::Array::Array {
 
     *flip = \&reverse;
 
-    sub to_hash {
-        my ($self) = @_;
-        Sidef::Types::Hash::Hash->new(@$self);
-    }
-
-    *to_h = \&to_hash;
-
-    sub to_a {
-        $_[0];
-    }
-
-    *to_array = \&to_a;
-
-    sub copy {
-        my ($self) = @_;
-        state $x = warn "[WARNING] Array.copy() is deprecated: use .clone() or .dclone() instead!\n";
-        $self->dclone;
-    }
-
     sub delete_first {
         my ($self, $obj) = @_;
 
@@ -3344,9 +3171,7 @@ package Sidef::Types::Array::Array {
             my $s;
 
             '['
-              . CORE::join(', ',
-                           map { ref($_) && ($s = UNIVERSAL::can($_, 'dump')) ? $s->($_) : defined($_) ? $_ : 'nil' } @$obj)
-              . ']';
+              . CORE::join(', ', map { ref($_) && ($s = UNIVERSAL::can($_, 'dump')) ? $s->($_) : ($_ // 'nil') } @$obj) . ']';
         };
 
         local *Sidef::Types::Array::Array::dump = $sub;
@@ -3357,7 +3182,36 @@ package Sidef::Types::Array::Array {
         Sidef::Types::String::String->new($_[0]->_dump);
     }
 
-    *to_s = \&dump;
+    *to_s   = \&dump;
+    *to_str = \&dump;
+
+    sub to_hash {
+        my ($self) = @_;
+        Sidef::Types::Hash::Hash->new(@$self);
+    }
+
+    *to_h = \&to_hash;
+
+    sub to_a {
+        $_[0];
+    }
+
+    *to_array = \&to_a;
+
+    sub to_set {
+        my ($self) = @_;
+        Sidef::Types::Set::Set->new(@$self);
+    }
+
+    sub to_bag {
+        my ($self) = @_;
+        Sidef::Types::Set::Bag->new(@$self);
+    }
+
+    sub to_matrix {
+        my ($self) = @_;
+        Sidef::Types::Array::Matrix->new(@$self);
+    }
 
     {
         no strict 'refs';
@@ -3387,18 +3241,6 @@ package Sidef::Types::Array::Array {
         *{__PACKAGE__ . '::' . '/'}   = \&div;
         *{__PACKAGE__ . '::' . '÷'}  = \&div;
         *{__PACKAGE__ . '::' . '...'} = \&to_list;
-
-        *{__PACKAGE__ . '::' . '++'} = sub {
-            my ($self, $obj) = @_;
-            CORE::push(@$self, $obj);
-            $self;
-        };
-
-        *{__PACKAGE__ . '::' . '--'} = sub {
-            my ($self) = @_;
-            CORE::pop(@$self);
-            $self;
-        };
     }
 
 };
