@@ -5,8 +5,6 @@ package Sidef::Types::Array::Matrix {
 
     use parent qw(Sidef::Types::Array::Array);
 
-    use overload q{""} => \&_dump;
-
     require List::Util;
 
     my %array_like = (
@@ -17,29 +15,11 @@ package Sidef::Types::Array::Matrix {
 
     sub new {
         my (undef, @rows) = @_;
-        bless [map { bless [@$_], 'Sidef::Types::Array::Array' } @rows];
+        bless [map { bless [@$_] } @rows];
     }
 
     *call = \&new;
 
-    sub is_square {
-        my ($self) = @_;
-
-        my $rows = $#{$self};
-        $rows < 0 and return Sidef::Types::Bool::Bool::TRUE;
-
-        ($rows == $#{$self->[0]})
-          ? Sidef::Types::Bool::Bool::TRUE
-          : Sidef::Types::Bool::Bool::FALSE;
-    }
-
-    sub size {
-        my ($self) = @_;
-        my $rows = $#{$self};
-        $rows < 0 and return ((Sidef::Types::Number::Number::ZERO) x 2);
-        (Sidef::Types::Number::Number->_set_uint($rows + 1), Sidef::Types::Number::Number->_set_uint($#{$self->[0]} + 1));
-    }
-
     sub build {
         my (undef, $n, $m, $block) = @_;
 
@@ -71,38 +51,22 @@ package Sidef::Types::Array::Matrix {
 
         $n = CORE::int($n);
 
+#<<<
         bless [
             map {
-                bless(
-                      [(Sidef::Types::Number::Number::ZERO) x ($_ - 1),
-                       Sidef::Types::Number::Number::ONE,
-                       (Sidef::Types::Number::Number::ZERO) x ($n - $_)
-                      ],
-                      'Sidef::Types::Array::Array'
-                     )
+                my $i = $_;
+                bless([map {
+                        $i == $_
+                            ? Sidef::Types::Number::Number::ONE
+                            : Sidef::Types::Number::Number::ZERO
+                } 1 .. $n], 'Sidef::Types::Array::Array')
               } 1 .. $n
         ];
+#>>>
     }
 
     *I = \&identity;
 
-    sub column {
-        my ($self, $n) = @_;
-        $n = CORE::int($n);
-        bless [map { $_->[$n] } @$self], 'Sidef::Types::Array::Array';
-    }
-
-    *col        = \&column;
-    *get_column = \&column;
-
-    sub row {
-        my ($self, $n) = @_;
-        $n = CORE::int($n);
-        bless [@{$self->[$n]}], 'Sidef::Types::Array::Array';
-    }
-
-    *get_row = \&row;
-
     sub zero {
         my (undef, $n, $m) = @_;
 
@@ -140,17 +104,18 @@ package Sidef::Types::Array::Matrix {
 
         $n = CORE::int($n);
 
+#<<<
         bless [
             map {
-                bless(
-                      [(Sidef::Types::Number::Number::ZERO) x ($_ - 1),
-                       $value,
-                       (Sidef::Types::Number::Number::ZERO) x ($n - $_)
-                      ],
-                      'Sidef::Types::Array::Array'
-                     )
+                my $i = $_;
+                bless([map {
+                            $i == $_
+                                ? $value
+                                : Sidef::Types::Number::Number::ZERO
+                    } 1 .. $n], 'Sidef::Types::Array::Array')
               } 1 .. $n
         ];
+#>>>
     }
 
     sub row_vector {
@@ -165,97 +130,45 @@ package Sidef::Types::Array::Matrix {
 
     *col_vector = \&column_vector;
 
-    sub _new_diagonal {
-        my (undef, @diag) = @_;
-        my $n = scalar(@diag);
-#<<<
-        __PACKAGE__->new([
-         map { [
-                (Sidef::Types::Number::Number::ZERO) x ($_ - 1),
-                shift(@diag),
-                (Sidef::Types::Number::Number::ZERO) x ($n - $_)
-            ] } 1 .. $n
-        ]);
-#>>>
-    }
-
-    sub _new_anti_diagonal {
-        my (undef, @diag) = @_;
-        my $n = scalar(@diag);
-#<<<
-        __PACKAGE__->new([
-         map { [
-                (Sidef::Types::Number::Number::ZERO) x ($n - $_),
-                shift(@diag),
-                (Sidef::Types::Number::Number::ZERO) x ($_ - 1)
-            ] } 1 .. $n
-        ]);
-#>>>
-    }
-
     sub diagonal {
-        my ($self) = @_;
-        ref($self) || goto &_new_diagonal;
-        bless [map { $self->[$_][$_] } 0 .. $#{$self}], 'Sidef::Types::Array::Array';
-    }
+        my (undef, @diag) = @_;
 
-    sub anti_diagonal {
-        my ($self) = @_;
-        ref($self) || goto &_new_anti_diagonal;
-        bless [map { my $row = $self->[$_]; $row->[$#{$row} - $_] } 0 .. $#{$self}], 'Sidef::Types::Array::Array';
+        my $n = scalar(@diag);
+
+#<<<
+        bless [
+            map {
+                my $i = $_;
+                bless([map {
+                    $i == $_
+                        ? shift(@diag)
+                        : Sidef::Types::Number::Number::ZERO
+                } 1 .. $n], 'Sidef::Types::Array::Array')
+            } 1 .. $n
+        ];
+#>>>
     }
 
     sub rows {
-        my ($self, @rows) = @_;
-        ref($self) && return $self->to_a;
+        my (undef, @rows) = @_;
+
         bless [map { bless([@$_], 'Sidef::Types::Array::Array') } @rows];
     }
 
-    *from_rows = \&rows;
-
     sub columns {
-        my ($self, @cols) = @_;
-
-        ref($self) && return $self->transpose->to_a;
+        my (undef, @cols) = @_;
 
         my $max = List::Util::max(map { scalar(@$_) } @cols);
 
         bless [
             map {
                 my $i = $_;
-                bless [map { $_->[$i] } @cols], 'Sidef::Types::Array::Array'
+                bless [map { $_->[$i] } @cols]
               } 0 .. $max - 1
         ];
     }
 
-    *cols         = \&columns;
-    *from_cols    = \&columns;
-    *from_columns = \&columns;
-
-    sub vector_rows {
-        my ($self) = @_;
-        bless([map { Sidef::Types::Array::Vector->new(@$_) } @$self], 'Sidef::Types::Array::Array');
-    }
-
-    *vec_rows = \&vector_rows;
-
-    sub vector_columns {
-        my ($self) = @_;
-        $self->transpose->vector_rows;
-    }
-
-    *vec_cols    = \&vector_columns;
-    *vec_columns = \&vector_columns;
-
-    sub neg {
-        my ($m1) = @_;
-        bless($m1->scalar_operator('neg'));
-    }
-
-    sub abs {
-        my ($m1) = @_;
-        bless($m1->scalar_operator('abs'));
-    }
+    *cols = \&columns;
 
     sub add {
         my ($m1, $m2) = @_;
@@ -323,88 +236,17 @@ package Sidef::Types::Array::Matrix {
         bless \@c;
     }
 
-    sub floor {
-        my ($self) = @_;
-
-        bless [
-            map {
-                bless [map { $_->floor } @$_], 'Sidef::Types::Array::Array'
-              } @$self
-        ];
-    }
-
-    sub ceil {
-        my ($self) = @_;
-
-        bless [
-            map {
-                bless [map { $_->ceil } @$_], 'Sidef::Types::Array::Array'
-              } @$self
-        ];
-    }
-
-    sub mod {
-        my ($m1, $m2) = @_;
-
-        if (exists $array_like{ref($m2)}) {
-            return $m1->sub($m2->mul($m1->div($m2)->floor));
-        }
-
-        bless($m1->scalar_operator('%', $m2));
-    }
-
-    sub and {
-        my ($m1, $m2) = @_;
-
-        if (exists $array_like{ref($m2)}) {
-            return bless($m1->wise_operator('&', $m2));
-        }
-
-        bless($m1->scalar_operator('&', $m2));
-    }
-
-    sub or {
-        my ($m1, $m2) = @_;
-
-        if (exists $array_like{ref($m2)}) {
-            return bless($m1->wise_operator('|', $m2));
-        }
-
-        bless($m1->scalar_operator('|', $m2));
-    }
-
-    sub xor {
-        my ($m1, $m2) = @_;
-
-        if (exists $array_like{ref($m2)}) {
-            return bless($m1->wise_operator('^', $m2));
-        }
-
-        bless($m1->scalar_operator('^', $m2));
-    }
-
-    sub sum {
-        my ($A, $block) = @_;
-        Sidef::Types::Array::Array->new([map { @$_ } @$A])->sum($block);
-    }
-
-    *sum_by = \&sum;
-
-    sub prod {
-        my ($A, $block) = @_;
-        Sidef::Types::Array::Array->new([map { @$_ } @$A])->prod($block);
-    }
-
-    *prod_by = \&prod;
-
     sub pow {
         my ($A, $pow) = @_;
 
         $pow = CORE::int($pow);
-        my $neg = ($pow < 0);
-        $pow = CORE::int(CORE::abs($pow));
 
-        return $A->inv if ($neg and $pow == 1);
+        my $neg = 0;
+
+        if ($pow < 0) {
+            $neg = 1;
+            $pow = -$pow;
+        }
 
 #<<<
         my $n = $#$A;
@@ -429,28 +271,6 @@ package Sidef::Types::Array::Matrix {
         $neg ? $B->inv : $B;
     }
 
-    sub powmod {
-        my ($A, $pow, $mod) = @_;
-
-        if ($pow->is_neg) {
-            $A   = $A->invmod($mod);
-            $pow = $pow->abs;
-        }
-
-        my $B = __PACKAGE__->identity($#{$A} + 1);
-
-        return $B->mod($mod) if $pow->is_zero;
-
-        while (1) {
-            $B   = $B->mul($A)->mod($mod) if $pow->is_odd;
-            $pow = $pow->rsft(Sidef::Types::Number::Number::ONE);
-            last if $pow->is_zero;
-            $A = $A->mul($A)->mod($mod);
-        }
-
-        $B->mod($mod);
-    }
-
     # Code translated from Wikipedia (+ minor tweaks):
     #   https://en.wikipedia.org/wiki/LU_decomposition#C_code_examples
 
@@ -560,20 +380,6 @@ package Sidef::Types::Array::Matrix {
     *inv     = \&invert;
     *inverse = \&invert;
 
-    sub invmod {
-        my ($self, $mod) = @_;
-
-        my $A = $self->inv;
-
-        foreach my $row (@$A) {
-            foreach my $i (0 .. $#{$row}) {
-                $row->[$i] = $row->[$i]->ratmod($mod);
-            }
-        }
-
-        return $A;
-    }
-
     sub determinant {
         my ($self) = @_;
 
@@ -618,23 +424,18 @@ package Sidef::Types::Array::Matrix {
 
             @m[$i, $r] = @m[$r, $i];
 
-            my $mr  = $m[$r];
-            my $mrj = $mr->[$j];
-
+            my $t = $m[$r][$j];
             foreach my $k (0 .. $cols - 1) {
-                $mr->[$k] = $mr->[$k]->div($mrj);
+                $m[$r][$k] = $m[$r][$k]->div($t);
             }
 
             foreach my $i (0 .. $rows - 1) {
 
                 $i == $r and next;
 
-                my $mr  = $m[$r];
-                my $mi  = $m[$i];
-                my $mij = $mi->[$j];
-
+                my $t = $m[$i][$j];
                 foreach my $k (0 .. $cols - 1) {
-                    $mi->[$k] = $mi->[$k]->sub($mij->mul($mr->[$k]));
+                    $m[$i][$k] = $m[$i][$k]->sub($t->mul($m[$r][$k]));
                 }
             }
 
@@ -652,12 +453,17 @@ package Sidef::Types::Array::Matrix {
 
         my $n = $#$self;
 
+#<<<
         my @I = map {
-            [(Sidef::Types::Number::Number::ZERO) x $_,
-             Sidef::Types::Number::Number::ONE,
-             (Sidef::Types::Number::Number::ZERO) x ($n - $_)
-            ]
+            my $i = $_;
+            [map {
+                $i == $_
+                    ? Sidef::Types::Number::Number::ONE
+                    : Sidef::Types::Number::Number::ZERO
+            } 0 .. $n]
         } 0 .. $n;
+#>>>
+
         my @A = map { [@{$self->[$_]}, @{$I[$_]}] } 0 .. $n;
 
         my $r = rref(\@A);
@@ -689,7 +495,7 @@ package Sidef::Types::Array::Matrix {
             my $prev_pivot = $pivot;
             $pivot = $m[$k][$k] // return Sidef::Types::Number::Number::ONE;
 
-            if ($pivot->is_zero) {
+            if ($pivot eq Sidef::Types::Number::Number::ZERO) {
                 my $i = List::Util::first(sub { $m[$_][$k] }, @r) // return Sidef::Types::Number::Number::ZERO;
                 @m[$i, $k] = @m[$k, $i];
                 $pivot = $m[$k][$k];
@@ -713,48 +519,6 @@ package Sidef::Types::Array::Matrix {
         bless($matrix->SUPER::transpose);
     }
 
-    *t   = \&transpose;
-    *not = \&transpose;
-
-    sub concat {
-        my ($m1, $m2) = @_;
-
-        if (exists $array_like{ref($m2)}) {
-
-            my $end = List::Util::min($#{$m1}, $#{$m2});
-
-            my @m3;
-            foreach my $i (0 .. $end) {
-                push @m3, bless [@{$m1->[$i]}, @{$m2->[$i]}], 'Sidef::Types::Array::Array';
-            }
-
-            return bless \@m3;
-        }
-
-        # Scalar concatenation
-        my @m3;
-        foreach my $i (0 .. $#{$m1}) {
-            push @m3, bless [@{$m1->[$i]}, $m2], 'Sidef::Types::Array::Array';
-        }
-
-        bless \@m3;
-    }
-
-    sub horizontal_flip {
-        my ($self) = @_;
-        bless [map { bless [reverse(@$_)], 'Sidef::Types::Array::Array' } @{$self}];
-    }
-
-    sub vertical_flip {
-        my ($self) = @_;
-        bless [reverse @{$self}];
-    }
-
-    sub flip {
-        my ($self) = @_;
-        bless [reverse map { bless [reverse(@$_)], 'Sidef::Types::Array::Array' } @{$self}];
-    }
-
     sub set_row {
         my ($A, $k, $row) = @_;
         $k = CORE::int($k);
@@ -796,24 +560,6 @@ package Sidef::Types::Array::Matrix {
     *row_len  = \&row_count;
     *row_size = \&row_count;
 
-    sub to_array {
-        my ($A) = @_;
-        Sidef::Types::Array::Array->new(@$A);
-    }
-
-    *to_a = \&to_array;
-
-    sub _dump {
-        "Matrix(\n  " . join(",\n  ", @{$_[0]}) . "\n)";
-    }
-
-    sub dump {
-        Sidef::Types::String::String->new($_[0]->_dump);
-    }
-
-    *to_s   = \&dump;
-    *to_str = \&dump;
-
     {
         no strict 'refs';
 
@@ -822,11 +568,7 @@ package Sidef::Types::Array::Matrix {
         *{__PACKAGE__ . '::' . '+'}  = \&add;
         *{__PACKAGE__ . '::' . '-'}  = \&sub;
         *{__PACKAGE__ . '::' . '/'}  = \&div;
-        *{__PACKAGE__ . '::' . '÷'}  = \&div;
-        *{__PACKAGE__ . '::' . '&'}  = \&and;
-        *{__PACKAGE__ . '::' . '|'}  = \&or;
-        *{__PACKAGE__ . '::' . '^'}  = \&xor;
-        *{__PACKAGE__ . '::' . '%'}  = \&mod;
+        *{__PACKAGE__ . '::' . '÷'} = \&div;
     }
 };
 
