@@ -68,6 +68,9 @@ package Sidef::Types::Glob::Pipe {
         my ($self) = @_;
         Sidef::Types::String::String->new("$self");
     }
+
+    *to_s   = \&dump;
+    *to_str = \&dump;
 }
 
 1;
